package asynchqueuereceiver;

import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * Der AsynchQueueReceiver besteht aus einer main Klasse, welche
 * eine oder mehrere Meldungen liest , asynchroon aus einer Warteschlange
 * <p>
 * Die Klasse bentigt den TextListener (TextListener.java)
 * <p>
 * Message Sender knnte sein: SenderToQueue.
 * Die Warteschlange wird als Parameter angegeben.
 *
 */

 public class AsynchQueueReceiver {

    /**
     * Main Methode
     *
     * @param args	Warteschlange z.B. meineWarteschlange
     */
    public static void main(String[] args) {
        String                  queueName = null;
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   queue = null;
        QueueReceiver           queueReceiver = null;
        TextListener            textListener = null;
        int                     exitResult = 0;

    	if (args.length != 1) {
    	    System.out.println("Usage: java AsynchQueueReceiver <queue_name>");
    	    System.out.println("Beispiel Usage: java AsynchQueueReceiver MeineWarteschlange");
            queueName = new String("MeineWarteschlange");
    	    //System.exit(1);
    	} else {
        queueName = new String(args[0]);}
        System.out.println("Name der Warteschlange " + queueName);

        try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                Session.AUTO_ACKNOWLEDGE);
            queue = Hilfsklassen.getQueue(queueName, queueSession);
        } catch (Exception e) {
            System.out.println("Problem beim Verbindungsaufbau: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
    	    System.exit(1);
        }

        /*
         * Kreiere den Receiver
         * Registriere den Messahe Listener (TextListener).
         * Starte Message Lieferung; der Listener zeigt die Meldungen an
         * Blockiere bis der Puublisher eine Kontrollmessage sendet
         * welche das Ende des Publishstream anzeigt.
         */
        try {
            queueReceiver = queueSession.createReceiver(queue);
            textListener = new TextListener();
            queueReceiver.setMessageListener(textListener);
            queueConnection.start();

            textListener.monitor.waitTillDone();
        } catch (JMSException e) {
            System.out.println("Exception: " + e.toString());
            exitResult = 1;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
        Hilfsklassen.exit(exitResult);
    }
}
