package bytemessages;

import java.io.*;
import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * BytesMessages
 * liest Textdatei, kreiert eine BytesMessage und liest diese Message
 * <p>
 * Die Textdatei wird ale Parameter eingegeben
 * <p>
 * Unrealistisches Beispiel
 * besser wre StreamMessage oder MapMessage
 */
public class BytesMessages {

    /**
     * Main Methode.
     *
     * @param args	Name der Textdatei
     */
    public static void main(String[] args) {
        String                  filename = null;
        FileInputStream         inStream = null;
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        BytesMessage            bytesMessage = null;
        int                     bytes_read = 0;
        final int               BUFLEN = 64;
        byte[]                  buf1 = new byte[BUFLEN];
        byte[]                  buf2 = new byte[BUFLEN];
        int                     length = 0;
        int                     exitResult = 0;

    	/*
    	 * lesen der Textdatei
    	 */
    	if (args.length != 1) {
    	    System.out.println("Usage: java BytesMessages <filename>");
            filename = "EingabeTextDatei.txt";
    	    //System.exit(1);
    	} else {filename = new String(args[0]);}
    	try {
            inStream = new FileInputStream(filename);
    	} catch (IOException e) {
    	    System.out.println("Probleme beim Lesen der Datei: " + e.toString());
            System.exit(1);
    	}

        try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                Session.AUTO_ACKNOWLEDGE);
    	} catch (Exception e) {
            System.out.println("Connection Problem: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
    	    System.exit(1);
    	}

        try {
            /*
             * Kreiere eine BytesMessage.
             * Lies die Bytes aus dem InputStream in einen Buffer
             * Konstruiere eine ByteMessage mit der writeBytes Methode
             * Die Methode garantiert, dass nur die Bytes aus der Datei in der
             * Message stehen, keine Zeichen aus dem Buffer
             */
            bytesMessage = queueSession.createBytesMessage();
            while ((bytes_read = inStream.read(buf1)) != -1) {
                bytesMessage.writeBytes(buf1, 0, bytes_read);
                System.out.println("Schreibe " + bytes_read
                    + " Bytes in die Message");
            }

            /*
             * Reset der Message (an den Anfang zum Lesen)
             * readBytes um den Inhalt zu extrahieren, in einen anderen Buffer
             * Caste die Bytes in char, zum Anzeigen
             */
            bytesMessage.reset();
            do {
                length = bytesMessage.readBytes(buf2);
                if (length != -1) {
                    System.out.println("Lese " + length
                        + " Bytes aus der Message: ");
                    for (int i = 0; i < length; i++) {
                        System.out.print((char)buf2[i]);
                    }
                }
                System.out.println();
            } while (length >= BUFLEN);
        } catch (JMSException e) {
            System.out.println("JMS Exception trat auf: " + e.toString());
            exitResult = 1;
        } catch (IOException e) {
            System.out.println("I/O Exception trat auf: " + e.toString());
            exitResult = 1;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
        Hilfsklassen.exit(exitResult);
    }
}
