package einfacherqueuereceiver;

// env.put(Context.INITIAL_CONTEXT_FACTORY,"com.sun.enterprise.naming.SerialInitContextFactory");
import java.util.*;
import javax.jms.*;
import javax.naming.*;
import com.sun.enterprise.naming.SerialInitContextFactory;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * EinfacherQueueReceiver Klasse bestehend aus main()
 * Diese liest eine oder mehrere Messages aus einer Warteschlange
 * synchron.
 * Das Partner Programm ist EinfacherQueueSender.
 * Die Warteschlange wird als Parameter bergeben
 */
public class EinfacherQueueReceiver {

    /**
     * Main Methode.
     *
     * @param args     die Warteschlange
     */
    public static void main(String[] args) {
        String                  queueName = null;
        Context                 jndiContext = null;
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   queue = null;
        QueueReceiver           queueReceiver = null;
        TextMessage             message = null;

        /*
         * Lies den queue Namen und zeig ihn an.
         */
        if (args.length != 1) {
            System.out.println("Usage: java EinfacherQueueReceiver " +
                "<queue-name>");
            queueName = "meineWarteschlange";
            //System.exit(1);
        } else {
        queueName = new String(args[0]);}
        System.out.println("[EinfacherQueueReceiver]Der Queue Name ist " + queueName);

        /*
         * Kreiere ein JNDI InitialContext Objekt falls noch keiner existiert.
         */
        try {
          jndiContext = new InitialContext();
        } catch (NamingException e) {
            System.out.println("[EinfacherQueueReceiver]JNDI Context konnte nicht generiert werden: "
                            + e.toString());
            System.exit(1);
        }

        /*
         * Look up Connection Factory Und Queue.
         * Falls eines oder beide nicht existieren wird das Programm abgebrochen
         */
        try {
            System.out.println("[EinfacherQueueReceiver]queueConnectionFactory");
            queueConnectionFactory = (QueueConnectionFactory)jndiContext.lookup("QueueConnectionFactory");

            System.out.println("[EinfacherQueueReceiver]queue");
            queue = (Queue) jndiContext.lookup(queueName);
        } catch (NamingException e) {
            System.out.println("[EinfacherQueueReceiver]JNDI Lookup fehlgeschlagen: "
                + e.toString());
            System.exit(1);
        }

        /*
         * Kreiere Connection.
         * Kreiere Session aus der Connection; false als Parameter bedeutet:
         *                                                keine Transaktion.
         * Kreiere Receiver,
         *      dann kann die Message Lieferung starten
         * Empfange alle Text Messages aus der Queue
         *      eine nicht-text Message zeigt das Ende des Message Stromes an
         * Close connection.
         */
        try {
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession =
                queueConnection.createQueueSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            queueReceiver = queueSession.createReceiver(queue);
            queueConnection.start();
            while (true) {
                Message m = queueReceiver.receive(1);
                if (m != null) {
                    if (m instanceof TextMessage) {
                        message = (TextMessage) m;
                        System.out.println("[EinfacherQueueReceiver]Lesen der Message: " +
                            message.getText());
                    } else {
                        break;
                    }
                }
            }
        } catch (JMSException e) {
            System.out.println("[EinfacherQueueReceiver]Es trat eine Exception auf: " +
                e.toString());
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {}
            }
        }
    }
}
