package einfacherqueuesender;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * Der EinfacherQueueSender besteht aus einermain Methode
 * die Nachrichten sendet, an eine Warteschlange
 *
 * Das Gegenstck ist der Receiver EinfacherQueueReceiver
 * Parameter sind die Warteschlange, die bereits in J2EE erfasst sein sollte
 * und die Anzahl Messages,die gesendet werden sollten
 */
import javax.jms.*;
import javax.naming.*;

public class EinfacherQueueSender {

    /**
     * Main method.
     *
     * @param args     Warteschlange
     *                 optional eine Anzahl Messages, die gesandt werden
     */
    public static void main(String[] args) {
        String                  queueName = null;
        Context                 jndiContext = null;
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   queue = null;
        QueueSender             queueSender = null;
        TextMessage             message = null;
        final int               NUM_MSGS;
        final String    MSG_TEXT = new String("Message Nummer ");

        if ( (args.length < 1) || (args.length > 2) ) {
            System.out.println("Usage: java EinfacherQueueSender " +
                "<queue-name> [<Anzahl-messages>]");
            queueName = "meineWarteschlange";
            //System.exit(1);
        } else {
        queueName = new String(args[0]); }
        System.out.println("Namen der Warteschlange " + queueName);
        if (args.length == 2){
            NUM_MSGS = (new Integer(args[1])).intValue();
        } else {
            NUM_MSGS = 10;
        }

        /*
         * Create a JNDI InitialContext object if none exists yet.
         */
        try {
            jndiContext = new InitialContext();
        } catch (NamingException e) {
            System.out.println("Der JNDI Context konnte nicht kreiert werden " +
                 e.toString());
            System.exit(1);
        }

        /*
         * Look up der Connection Factory und Queue.
         * Falls eine oder beide nicht existieren, wird abgebrochen.
         */
        try {
            queueConnectionFactory = (QueueConnectionFactory)
                jndiContext.lookup("QueueConnectionFactory");
            queue = (Queue) jndiContext.lookup(queueName);
        } catch (NamingException e) {
            System.out.println("JNDI Lookup schlug fehl: " +
                e.toString());
            System.exit(1);
        }

        /*
         * Programmlogik:
         * Kreiere connection Objekt
         * Kreiere session Objekt aus dem connection Objekt;
         *      false Parameter: keine Transaktion
         * Kreiere Sender und Text Message.
         * Sende Messages
         * Sende end-of-messages Message.
         * close connection Objekt.
         */
        try {
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession =
                queueConnection.createQueueSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            queueSender = queueSession.createSender(queue);
            message = queueSession.createTextMessage();
            for (int i = 0; i < NUM_MSGS; i++) {
                message.setText(MSG_TEXT + " " + (i + 1));
                System.out.println("Message " + i +" : "+
                    message.getText());
                queueSender.send(message);
            }

            /*
             * Senden einer non-text control message zeigt das Ende der
             * Messages.
             */
            queueSender.send(queueSession.createMessage());
        } catch (JMSException e) {
            System.out.println("Es trat eine Exception auf: " +
                e.toString());
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {}
            }
        }
    }
}
