package einfachertopicpublisher;

import javax.jms.*;
import javax.naming.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * EInfacherTopicPublisher : besteht aus einer main Methode, die
 * Messages an ein Topic publiziert
 *
 * Partnetprogramm: EInfacherTopicSubscriber.
 *
 * Das Topic wird als Parameter bergeben. Es muss zuerst registriert werden
 * Standardmssig sendet das Programm eine einfache Meldung
 * Sie knnen aber auch eine Anzahl Messages als Parameter 2 angeben
 */
public class EinfacherTopicPublisher {

    /**
     * Main Methode.
     *
     * @param args     topic
     *                 optional: Anzahl Messages
     */
    public static void main(String[] args) {
        String                  topicName = null;
        Context                 jndiContext = null;
        TopicConnectionFactory  topicConnectionFactory = null;
        TopicConnection         topicConnection = null;
        TopicSession            topicSession = null;
        Topic                   topic = null;
        TopicPublisher          topicPublisher = null;
        TextMessage             message = null;
        final int               NUM_MSGS;
        final String    MSG_TEXT = new String("Einfache Text Message");

        if ( (args.length < 1) ) {
            System.out.println("Usage: java " +
                "EinfacherTopicPublisher <topic-name> " +
                "[<anzahl-messages>]");
            topicName = "meinTopic";
            //System.exit(1);
        } else {
        topicName = new String(args[0]); }
        System.out.println("Der Name des Topics ist " + topicName);
        if (args.length == 2){
            NUM_MSGS = (new Integer(args[1])).intValue();
        } else {
            NUM_MSGS = 1;
        }
        System.out.println("Kreieren des JDNI Initial Contexts");
        /*
         * Kreieren eines JNDI InitialContext Objekt falls noch keines existiert
         */
        try {
            jndiContext = new InitialContext();
        } catch (NamingException e) {
            System.out.println("JNDI Context konnte nicht kreiert werden: " +
                            e.toString());
            System.exit(1);
        }

        /*
         * Look up der Connection Factory und des Topics.
         * Falls keines existiert, wird abgebrochen
         */
        System.out.println("Lookup der Connection Factory und des Topics");
        try {
            topicConnectionFactory = (TopicConnectionFactory)
                jndiContext.lookup("TopicConnectionFactory");
            topic = (Topic) jndiContext.lookup(topicName);
        } catch (NamingException e) {
            System.out.println("JNDI Lookup schlug fehl: " +
                e.toString());
            System.exit(1);
        }

        /*
         * Ablauf:
         * kreiere connection Objekt.
         * kreiere session Objekt aus der connection;
         *        'false' bedeutet: keine Transaktion
         * kreiere Publisher und Text Message.
         * sende Messages
         * schliesse connection Objekt.
         */
        try {
            System.out.println("createTopicConnection");
            topicConnection =
                topicConnectionFactory.createTopicConnection();
            System.out.println("createTopicSession");
            topicSession =
                topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            System.out.println("createPublisher");
            topicPublisher = topicSession.createPublisher(topic);
            System.out.println("createTextMessage");
            message = topicSession.createTextMessage();
            for (int i = 0; i < NUM_MSGS; i++) {
                message.setText(MSG_TEXT + " " + (i + 1));
                System.out.println("Message wird publiziert: " +
                    message.getText());
                System.out.println("publish()");
                topicPublisher.publish(message);
            }
        } catch (JMSException e) {
            System.out.println("Es trat eine Exception auf: " +
                e.toString());
        } finally {
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException e) {}
            }
        }
    }
}
