package einfachertopicsubscriber;

import javax.jms.*;
import javax.naming.*;
import java.io.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

 /**
 * EinfacherTopicSubscriber
 * empfngt eine oder mehrere Text Messages aus dem Topic
 * mittels asynchroner Kommunikation.
 * Es wird ein Listener eingesetzt: TextListener.
 * Das Programm gehrt zu : EinfacherTopicPublisher
 * und der Hilfsklasse TextListener
 *
 * Parameter ist das Topic
 *
 * Abbruch / Beenden des Programms mit Q oder q
 */
public class EinfacherTopicSubscriber {

    /**
     * Main Methode.
     *
     * @param args     Topic
     */
    public static void main(String[] args) {
        String                  topicName = null;
        Context                 jndiContext = null;
        TopicConnectionFactory  topicConnectionFactory = null;
        TopicConnection         topicConnection = null;
        TopicSession            topicSession = null;
        Topic                   topic = null;
        TopicSubscriber         topicSubscriber = null;
        TextListener            topicListener = null;
        TextMessage             message = null;
        InputStreamReader       inputStreamReader = null;
        char                    answer = '\0';

        /*
         * Lies den Namen aus der Kommandozeile
         */
        if (args.length != 1) {
            System.out.println("Usage: java EinfacherTopicSubscriber "
                + "<topic-name>");
            //System.exit(1);
            topicName = "MeinTopic";
        } else {   topicName = new String(args[0]);}
        System.out.println("Name des Topics : " + topicName);

        /*
         * kreiere einen JNDI InitialContext falls noch keiner existiert
         */
        try {
            jndiContext = new InitialContext();
        } catch (NamingException e) {
            System.out.println("Der JNDI Context konnte nicht kreiert werden: " +
                        e.toString());
            System.exit(1);
        }

        /*
         * Look Up des Connection Factory Objekts und des Topic.
         */
        try {
            topicConnectionFactory = (TopicConnectionFactory)
                jndiContext.lookup("TopicConnectionFactory");
            topic = (Topic) jndiContext.lookup(topicName);
            //NamingEnumeration ne = jndiContext.getNameInNamespace();
            System.out.println("Inhalt des Contexts:"+jndiContext.listBindings()
        } catch (NamingException e) {
            System.out.println("JNDI Lookup schlug fehl: "
                + e.toString());
            System.exit(1);
        }

        /*
         * Ablauf:
         * kreiere connection.
         * kreiere session mit der connection;
         *    'false' bedeutet: keine Transaktion
         * kreiere subscriber.
         * registriere message listener (TextListener).
         * empfange Text Messages vom Topic.
         * Nachdem alle Meldungen emfangen wurden: Q oder quit.
         * schliesse connection.
         */
        try {
            topicConnection =
                topicConnectionFactory.createTopicConnection();
            ConnectionMetaData cmd = topicConnection.getMetaData();
            System.out.println("Comnnection Metadata : ");
            System.out.println("                       JMSMajorVersion="+cmd.getJMSMajorVersion());
            System.out.println("                       JMSMinorVersion="+cmd.getJMSMinorVersion());
            System.out.println("                       JMSProviderName="+cmd.getJMSProviderName());
            System.out.println("                       JMSVersion="+cmd.getJMSVersion());
            System.out.println("                       JMSXPropertyVersion="+cmd.getJMSXPropertyNames());
            System.out.println("                       JMSXProviderVersion="+cmd.getJMSXPropertyNames());

            topicSession =
                topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            topicSubscriber = topicSession.createSubscriber(topic);
            topicListener = new TextListener();
            topicSubscriber.setMessageListener(topicListener);
            topicConnection.start();
            System.out.println("Beenden des Programms mit Q oder q, "
                + "und <return>");
            inputStreamReader = new InputStreamReader(System.in);
            while (!((answer == 'q') || (answer == 'Q'))) {
                try {
                    answer = (char) inputStreamReader.read();
                } catch (IOException e) {
                    System.out.println("I/O Exception: "
                        + e.toString());
                }
            }
        } catch (JMSException e) {
            System.out.println("Es trat eine Exception auf: "
                + e.toString());
        } finally {
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException e) {}
            }
        }
    }
}
