package messageformate;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */
import javax.jms.*;

/**
 * MessageFormate kreiert, liest und schreibt Messages in unterschiedlichen
 * Formaten:  BytesMessage, TextMessage, MapMessage, StreamMessage u
 * und ObjectMessage.
 * Die Messages werden nicht echt gesendet
 */
public class MessageFormate {

    /**
     * Main method.  kein Argument.
     */
    public static void main(String[] args) {
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        BytesMessage            bytesMessage = null;
        byte[]                  byteData = {-128, 127, -1, 0, 1, -64, 64};
        int                     length = 0;
        byte[]                  inByteData = new byte[7];
        TextMessage             textMessage = null;
        String                  msgText = "Eine einfache Text Message.";
        MapMessage              mapMessage = null;
        StreamMessage           streamMessage = null;
        ObjectMessage           objectMessage = null;
        String                  object = "Ein String ist ein Objekt.";
        int                     exitResult = 0;

    	try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
    	    queueConnection =
    	        queueConnectionFactory.createQueueConnection();
    	    queueSession = queueConnection.createQueueSession(false,
    	        Session.AUTO_ACKNOWLEDGE);
    	} catch (Exception e) {
            System.out.println("Connection Problem: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
    	    System.exit(1);
    	}

        try {
    	    /*
    	     * BytesMessage :
             * kreieren, schreiben in ein Arrays von Bytes (signed 8-Bit Ints)
    	     * Message auf Reading setzen.
             * Bytes in ein zweites Array lesen
    	     * BytesMessages sind ByteStreams, ohne Semantik / MIME Type
    	     */
    	    bytesMessage = queueSession.createBytesMessage();
    	    bytesMessage.writeBytes(byteData);
    	    bytesMessage.reset();
            length = bytesMessage.readBytes(inByteData);
            System.out.println("[ByteMessage]Laenge der BytesMessage " + length
                + "\t\t Bytes:");
            for (int i = 0; i < length; i++) {
                System.out.print("  " + inByteData[i]);
            }
    	    System.out.println();

    	    /*
    	     * TextMessage:
    	     * Zeichenkette
    	     */
    	    textMessage = queueSession.createTextMessage();
    	    textMessage.setText(msgText);
            System.out.println("[TextMessage]Lesen der TextMessage:");
            System.out.println("\t\t " + textMessage.getText());

    	    /*
    	     * MapMessage
    	     * eine MapMessage enthlt Namen/Werte Paare.
    	     * Der Name ist ein String; der Wert kann von unterschiedlichem
             * Typus sein.
    	     */
    	    mapMessage = queueSession.createMapMessage();
    	    mapMessage.setString("Message Typ", "Map");
    	    mapMessage.setInt("Ein Integer", 3456);
    	    mapMessage.setDouble("Ein Double", 1.23456789);
            System.out.println("[MapMessage]Lesen der Message in irgend einer Reihenfolge"
                + ", die sich von der urspruenglichen unterscheiden kann:");
            System.out.println("\t\t Typ: "
                + mapMessage.getString("Message Typ"));
            System.out.println("\t\t Double: "
                + mapMessage.getDouble("Ein Double"));
            System.out.println("\t\t Integer: "
                + mapMessage.getInt("Ein Integer"));

    	    /*
    	     * StreamMessage
    	     * eine StreamMessage kann unterschiedliche Datentypen enthalten
    	     * Die Lese-Reihenfolge ist identisch mit jener, in der der Stream
    	     * generiert wurde.
    	     */
    	    streamMessage = queueSession.createStreamMessage();
    	    streamMessage.writeString("Stream Message");
    	    streamMessage.writeDouble(123.456789e222);
    	    streamMessage.writeInt(223344);
    	    streamMessage.reset();
            System.out.println("[StreamMessage]Lesen der StreamMessage so"
                + " wie sie generiert wurde:");
            System.out.println("\t\t String: "
                + streamMessage.readString());
            System.out.println("\t\t Double: "
                + streamMessage.readDouble());
            System.out.println("\t\t Integer: "
                + streamMessage.readInt());

    	    /*
    	     * ObjectMessage (String Objekt)
    	     * Eine ObjectMessage kann beliebige Java Objekte enthalten
             * Sie drfen das Casten nicht vergessen
    	     */
    	    objectMessage = queueSession.createObjectMessage();
    	    objectMessage.setObject(object);
            System.out.println("[ObjectMessage]Lesen einer ObjectMessage:");
            System.out.println("\t\t " + (String) objectMessage.getObject());

    	} catch (JMSException e) {
    	    System.out.println("Eine Exception trat auf: " + e.toString());
    	    exitResult = 1;
    	} finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {
    	            exitResult = 1;
                }
    	    }
    	}
    	Hilfsklassen.exit(exitResult);
    }
}
