package messagekonversion;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import javax.jms.*;

/**
 * MessageKonversion
 * Konversion von StreamMessage und BytesMessage.
 * <p>
 * Konversionen in StreamMessages:  Sie knnen Daten als String schreiben
 * und als Int lesen und umgekehrt.
 * Die Methode clearBody() lscht eine Message, so dass sie neu be/geschrieben
 * werden kann.
 * <p>
 * Konnversion in BytesMessage:
 * String in Byte geht nicht
 * <p>
 * Reset() versetzt den Message Body in read-only
 *
 */
 public class MessageConversion {

    /**
     * Main method.  kein Argument
     */
    public static void main(String[] args) {
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        BytesMessage            bytesMessage = null;
        StreamMessage           streamMessage = null;
        int                     exitResult = 0;

        try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                Session.AUTO_ACKNOWLEDGE);
    	} catch (Exception e) {
            System.out.println("Connection Problem: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
    	    System.exit(1);
    	}

        try {
            /*
             * StreamMessage: schreibe unterschiedliche Datentypen in den Stream
             * Reset und lies als String
             */
            streamMessage = queueSession.createStreamMessage();
    	    streamMessage.writeBoolean(false);
    	    streamMessage.writeDouble(123.456789e222);
    	    streamMessage.writeInt(223344);
    	    streamMessage.writeChar('q');
            streamMessage.reset();
            System.out.println("[StreamMessage]Lesen der StreamMessage"
                + " mit unterschiedlichen Datentypen als String:");
            System.out.println("\t\t Boolean: " + streamMessage.readString());
            System.out.println("\t\t Double: " + streamMessage.readString());
            System.out.println("\t\t Int: " + streamMessage.readString());
            System.out.println("\t\t Char: " + streamMessage.readString());

            /*
             * Lschen des StreamMessage Body und  schreiben der Strings
             * Reset und lies die Werte, aber nicht als Strings
             */
            streamMessage.clearBody();
            streamMessage.writeString("true");
            streamMessage.writeString("123.456789e111");
            streamMessage.writeString("556677");
            // kein char:  String zu char Konversion geht nicht
            streamMessage.reset();
            System.out.println("[StreamMessage]Lesen der StreamMessage Strings "
                + " als andere Datentypen:");
            System.out.println("\t\t Boolean: " + streamMessage.readBoolean());
            System.out.println("\t\t Double: " + streamMessage.readDouble());
            System.out.println("\t\t Int: " + streamMessage.readInt());

            /*
             * BytesMessage
             * schreiben unterschiedlicher Datentypen in die Message.
             */
            bytesMessage = queueSession.createBytesMessage();
    	    bytesMessage.writeBoolean(false);
    	    bytesMessage.writeDouble(123.456789e22);
    	    bytesMessage.writeInt(778899);
    	    bytesMessage.writeInt(0x7f800000);
    	    bytesMessage.writeChar('z');

    	    /*
    	     * Reset der Message und lesen dieser Werte
    	     * sofern Konversionen mglich sind
    	     */
            bytesMessage.reset();
            System.out.println("[ByteMessage]Lesen der BytesMessages "+
                " bestehend aus unterschiedlichen Datentypen:");
            System.out.println("\t\t Boolean: " + bytesMessage.readBoolean());
            System.out.println("\t\t Double: " + bytesMessage.readDouble());
            System.out.println("\t\t Int: " + bytesMessage.readInt());
            System.out.println("\t\t Float: " + bytesMessage.readFloat());
            System.out.println("\t\t Char: " + bytesMessage.readChar());
        } catch (JMSException e) {
            System.out.println("Eine JMS Exception trat auf: " + e.toString());
            exitResult = 1;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
        Hilfsklassen.exit(exitResult);
    }
}
