package sendertoqueue;

import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */
/**
 * SenderToQueue ist ein einfacher Sender einer vordefinierten Message an
 * eine Queue, die als Parameter bergeben wird.
 * <p>
 * Gegenstueck ist der SynchQueueReceiver oder AsynchQueueReceiver.
 *
 * Geben Sie eine Warteschlange auf dem DOS Prompt ein oder als Parameter
 * Ohne Angane der Anzahl Meldungen wird einfach eine Meldung geschickt und
 * das Programm danach beendet.
 * Die Meldungen werden von der Queue noch eine Weile aufbewahrt.
 */

 public class SenderToQueue {

    /**
     * Main method.
     *
     * @param args	Warteschlange, in die geschrieben wird
     *                  Optional: Anzahl Meldungen, die generiert werden sollen
     */
    public static void main(String[] args) {
        String                  queueName = null;
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   queue = null;
        QueueSender             queueSender = null;
        TextMessage             message = null;
        final int               NUM_MSGS;
        final String            MSG_TEXT = new String("Das ist Meldung Nummer ");
        int                     exitResult = 0;

    	if ( (args.length < 1) || (args.length > 2) ) {
    	    System.out.println("Usage: java SenderToQueue <queue_name> [<anzahl_messages>]");
    	    System.out.println("Beispiel Usage: java SenderToQueue MeineWarteSchlange");
    	    //System.exit(1);
            queueName = new String("MeineWarteschlange");
    	} else {
        queueName = new String(args[0]);}
        System.out.println("Die Warteschlange heisst " + queueName);
        if (args.length == 2){
            NUM_MSGS = (new Integer(args[1])).intValue();
        } else {
            NUM_MSGS = 10;
        }
        System.out.println("Die Anzahl Meldungen ist " + NUM_MSGS);

        try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
            queueConnection =
                queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                Session.AUTO_ACKNOWLEDGE);
            queue = Hilfsklassen.getQueue(queueName, queueSession);
    	} catch (Exception e) {
            System.out.println("Problem beim Verbindungsaufbau: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
    	    System.exit(1);
    	}

        /*
         * Kreiere den Sender.
         * Kreiere die Textmessage
         * Sende die Meldungen
         * Sende Ende Message
         * Schliesse die Verbindung
         */
        try {
            queueSender = queueSession.createSender(queue);
            message = queueSession.createTextMessage();
            for (int i = 0; i < NUM_MSGS; i++) {
                message.setText(MSG_TEXT + " " + (i + 1));
                System.out.println("Message "+i+" : " + message.getText());
                queueSender.send(message);
            }

            // Ende
            queueSender.send(queueSession.createMessage());
        } catch (JMSException e) {
            System.out.println("Eine Ausnahme trat auf: " + e.toString());
            exitResult = 1;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
    	Hilfsklassen.exit(exitResult);
    }
}
