package syncqueuereceiver;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */
import javax.jms.*;

/**
 * Der SynchQueueReceiver besteht aus einer main Methode
 * welche eine oder mehrere Meldungen aus der Queue liest
 * mittels synchronem Transport.
 *
 * Das Gegenstck ist der SenderToQueue
 * Geben Sie ein Warteschlange an, jene die Sie lesen wollen.
 * <p>
 * Zustzlich bentigen Sie die Hilfsklassen Methoden
 */
public class SynchQueueReceiver {

    /**
     * Main
     *
     * @param args	Warteschlange
     */
    public static void main(String[] args) {
        String                  queueName = null;
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   queue = null;
        QueueReceiver           queueReceiver = null;
        TextMessage             message = null;
        int                     exitResult = 0;

    	/*
    	 * Lies Warteschlange aus args und zeige diese an
    	 */
    	if (args.length != 1) {
    	    System.out.println("Usage: java SynchQueueReceiver <queue_name>");
    	    System.out.println("Beispiel Usage: java SynchQueueReceiver MeineWarteschlange");
    	    //System.exit(1);
            queueName = new String("MeineWartesschlange");
    	} else {
    	queueName = new String(args[0]); }
    	System.out.println("Name der Warteschlange: " + queueName);

        /*
         * Connection Factory bestimmen
         * Verbindung aufbauen (Connection Objekt)
         * Session aus dem Connection Objekt bestimmen
         * Warteschlangenname
         */
    	try {
    	    queueConnectionFactory =
    	        Hilfsklassen.getQueueConnectionFactory();
    	    queueConnection =
    	        queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                Session.AUTO_ACKNOWLEDGE);
            queue = Hilfsklassen.getQueue(queueName, queueSession);
    	} catch (Exception e) {
            System.out.println("Problem beim Verbindungsaufbau: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
    	    System.exit(1);
    	}

        /*
         * Kreiere den Empfnger und dann empfange.
	 * Empfange alle Texte, bis die Warteschlange leer ist
	 * eine nicht-Text Message zeigt das Ende des Message Streams an
         * Schliesse das connection Objekt und Ende
         */
        try {
            queueReceiver = queueSession.createReceiver(queue);
            queueConnection.start();
	    while (true) {
		Message m = queueReceiver.receive();
		if (m instanceof TextMessage) {
		    message = (TextMessage) m;
		    System.out.println("Lesen der Meldung: " + message.getText());
		} else {
                    // nicht-Text Message zeigt das Ende der bertragung
 		    break;
		}
	    }
        } catch (JMSException e) {
            System.out.println("Eine Exception trat auf: " + e.toString());
            exitResult = 1;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
    	Hilfsklassen.exit(exitResult);
    }
}
